class WidgetFinder:
    """Semi hack to avoid breaking things.
    Could also be an interface for widgetfinders.
    """
    def find(self, node, REQUEST=None):
        """Find widget id for node.
        """
        pass
    
class NodeNameWidgetFinder:
    """Finds widgets, based on NodeName and isNodeSupported.
    """
    def __init__(self):
        """Create a widgetfinder.
        """
        self.widget = {}
        
    def find(self, node, REQUEST=None):
        """Try to find widget in list of widget levels.

        node -- node to find widget for
        REQUEST -- needs to be passed to make Zope authentication play nice
        Result -- the id of the first widget found, or None if not found
        """
        widgetlist = self.widgets.get(node.getNodeName(), [])
        for widget in widgetlist:
            # return the first widget that says it's the right one
            if widget._isNodeSupported(node, REQUEST):
                return widget.id
        return None # didn't find it
    
    def addWidget(self, widget):
        """Add a widget to the widgetfinder.

        widget -- the widget to add
        """
        widgets = self.widgets
    
        widgetlist = widgets.get(widget.nodename, [])
        widgetlist.append(widget)
        widgets[widget.nodename] = widgetlist
        self.widgets = widgets
        
    def clear(self):
        """Clear all widgets.
        """
        self.widgets = {}

class NodeWidgetFinder:
    """Finds widgets for specific nodes.
    """
    def __init__(self):
        """Create a widgetfinder.
        """
        self.widgetids = {}
        
    def find(self, node, REQUEST=None):
        """Try to find widget in list of widget levels.

        node -- node to find widget for
        REQUEST -- needs to be passed to make Zope authentication play nice
        Result -- the id of the first widget found, or None if not found
        """
        return self.widgetids.get(node.absolute_url(), None)
    
    def addWidget(self, widget, node):
        """Add a widget for a specific node to the widgetfinder.

        widget -- the widget to add
        node -- the node that this widget is for
        """
        self.widgetids[node.absolute_url()] = widget.id
        self.widgetids = self.widgetids
        
    def clear(self):
        """Clear all widgets..
        """
        self.widgetids = {}
