from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent
   
from OFS.ObjectManager import ObjectManager
from OFS import SimpleItem
from AccessControl.Role import RoleManager
from OFS.PropertyManager import PropertyManager

from Widget import Widget

import string

class ZWidget(
    ObjectManager,
    PropertyManager,
    RoleManager,
    SimpleItem.Item,
    Widget
    ):

    meta_type = "XMLWidget"

    # the various tabs
    manage_options=(
        {'label':'Contents', 'action':'manage_main'},
        {'label':'Settings', 'action':'manage_settingsForm'},
        {'label':'Properties', 'action':'manage_propertiesForm'},
        {'label':'Security', 'action':'manage_access'},
    )

    __ac_permissions__ = (
        ('Change XMLWidgets',
         ('manage_settings',)),
        
        ('View',
         ('isNodeSupported',)),
        )
    
    # settings screen
    manage_settingsForm = HTMLFile('widgetSettings', globals()) 
    
    def __init__(self, id, title, nodename):
        """Create a new widget
        """
        ZWidget.inheritedAttribute("__init__")(self, nodename)
        self.id = id
        self.title = title
        self.parent_widget_id = None
        
    def manage_settings(self, REQUEST=None):
        """Handle result from edit action.
        REQUEST - contains other data
        """
        if REQUEST is not None:
            self.nodename = REQUEST.nodename
            parent_widget_id = string.strip(REQUEST.parent_widget_id)
            if parent_widget_id != "":
                self.parent_widget_id = parent_widget_id
            else:
                self.parent_widget_id = None
                
            message = "Properties changed"
            return self.manage_settingsForm(self, REQUEST,
                                            manage_tabs_message=message)

    def _inheritContext(self):
        """Get widget in acquisition context faking (single) inheritance.

        Uses acquisition hack to fake inheritance.
        """
        parent_widgets = []

        # find all parent_widgets
        widget = self
        while (hasattr(widget, 'parent_widget_id') and
               widget.parent_widget_id):
            # get parent
            widget = getattr(self.aq_inner.aq_parent, widget.parent_widget_id)
            # append parent to list
            parent_widgets.append(widget)

        # now get this widget in context of all parents
        parent_widgets.reverse()
        context_widget = self
        for widget in parent_widgets:
            context_widget = context_widget.__of__(widget)
        return context_widget
    
# form to add the Widget to a folder
manage_addWidgetForm = HTMLFile('widgetAdd', globals())

# adds Widget to folder
def manage_addWidget(self, id, title='', REQUEST=None):
    """Add widget to folder.
    """
    widget = ZWidget(id, title, REQUEST.nodename)
    
    self._setObject(id, widget)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
